/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Copyright (C) 2003 Hiroyuki Ikezoe
 *  Copyright (C) 2003 Takuro Ashie
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: kz-bookmark.h,v 1.6 2003/10/25 20:46:19 makeinu Exp $
 */

#ifndef __KZ_BOOKMARK_H__
#define __KZ_BOOKMARK_H__

#include <glib-object.h>

G_BEGIN_DECLS

#define KZ_TYPE_BOOKMARK		(kz_bookmark_get_type ())
#define KZ_BOOKMARK(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), KZ_TYPE_BOOKMARK, KzBookmark))
#define KZ_BOOKMARK_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass),  KZ_TYPE_BOOKMARK, KzBookmarkClass))
#define KZ_IS_BOOKMARK(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), KZ_TYPE_BOOKMARK))
#define KZ_IS_BOOKMARK_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass),  KZ_TYPE_BOOKMARK))
#define KZ_BOOKMARK_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj),  KZ_TYPE_BOOKMARK, KzBookmarkClass))

#define KZ_TYPE_BOOKMARK_TYPE		(kz_bookmark_type_get_type())

typedef struct _KzBookmark      KzBookmark;
typedef struct _KzBookmarkClass KzBookmarkClass;

typedef enum
{
	KZ_BOOKMARK_NORMAL,
	KZ_BOOKMARK_FOLDER,
	KZ_BOOKMARK_PURE_FOLDER,
	KZ_BOOKMARK_SEPARATOR,
	KZ_BOOKMARK_N_TYPES
} KzBookmarkType;

typedef enum
{
	KZ_BOOKMARK_EDITABLE_FLAG    = 1 << 0,
	KZ_BOOKMARK_FOLDED_FLAG      = 1 << 1,
	KZ_BOOKMARK_REMOTE_FLAG      = 1 << 2,
	KZ_BOOKMARK_SMART_FLAG       = 1 << 3,
	KZ_BOOKMARK_HAS_LOCATION     = 1 << 4  /* has location of bookmark file */
} KzBookmarkFlag;

typedef enum
{
	KZ_BOOKMARK_NONE,
	KZ_BOOKMARK_SORT_LAST_MODIFIED
} KzBookmarkSortType;

struct _KzBookmark
{
	GObject        parent;
	KzBookmarkType type;
	KzBookmarkFlag flags;
};

struct _KzBookmarkClass
{
	GObjectClass parent_class;

	void (*insert_child)       (KzBookmark *bookmark,
				    KzBookmark *child,
				    KzBookmark *sibling);
	void (*remove_child)       (KzBookmark *bookmark,
				    KzBookmark *child);
	void (*move_child)         (KzBookmark *bookmark,
				    KzBookmark *parent,
				    KzBookmark *sibling);
	void (*children_reordered) (KzBookmark *boomark);
};


GType        kz_bookmark_type_get_type     (void);
GType        kz_bookmark_get_type          (void);
KzBookmark  *kz_bookmark_new               (void);
KzBookmark  *kz_bookmark_new_with_attrs    (const gchar *title,
					    const gchar *uri,
					    const gchar *description);
KzBookmark  *kz_bookmark_pure_folder_new   (void);
KzBookmark  *kz_bookmark_separator_new     (void);

/* set/get properties */
/*
 * "title" means user defined title.
 * "document_title" is the original document title.
 * "location" means location of bookmark file itself.
 */
const gchar *kz_bookmark_get_link          (KzBookmark  *bookmark);
const gchar *kz_bookmark_get_title         (KzBookmark  *bookmark);
const gchar *kz_bookmark_get_document_title(KzBookmark  *bookmark);
const gchar *kz_bookmark_get_description   (KzBookmark  *bookmark);
const gchar *kz_bookmark_get_location      (KzBookmark  *bookmark);
guint        kz_bookmark_get_last_modified (KzBookmark  *bookmark);

void         kz_bookmark_set_link          (KzBookmark  *bookmark,
					    const gchar *uri);
void         kz_bookmark_set_title         (KzBookmark  *bookmark,
					    const gchar *title);
void         kz_bookmark_set_document_title(KzBookmark  *bookmark,
					    const gchar *doc_title);
void         kz_bookmark_set_description   (KzBookmark  *bookmark,
					    const gchar *description);
void         kz_bookmark_set_location      (KzBookmark  *bookmark,
					    const gchar *location);
void         kz_bookmark_set_last_modified (KzBookmark  *bookmark,
					    guint        time);

/* separator */
gboolean     kz_bookmark_is_separator      (KzBookmark  *bookmark);

/* folder interface */
gboolean     kz_bookmark_is_folder         (KzBookmark  *bookmark);
gboolean     kz_bookmark_is_pure_folder    (KzBookmark  *bookmark);
gboolean     kz_bookmark_get_folded        (KzBookmark  *bookmark);
void         kz_bookmark_set_folded        (KzBookmark  *bookmark,
					    gboolean     folded);
void         kz_bookmark_insert_before     (KzBookmark  *bookmark,
					    KzBookmark  *child,
					    KzBookmark  *sibling);
void         kz_bookmark_append            (KzBookmark  *bookmark,
					    KzBookmark  *child);
void         kz_bookmark_prepend           (KzBookmark  *bookmark,
					    KzBookmark  *child);
void         kz_bookmark_remove            (KzBookmark  *bookmark,
					    KzBookmark  *child);
void         kz_bookmark_remove_all        (KzBookmark  *bookmark);
KzBookmark  *kz_bookmark_get_parent        (KzBookmark  *bookmark);
GList       *kz_bookmark_get_children      (KzBookmark  *bookmark);
void         kz_bookmark_sort              (KzBookmark  *bookmark,
					    KzBookmarkSortType type);


G_END_DECLS

#endif /* __KZ_BOOKMARK_H__ */
