/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Copyright (C) 2003 Hiroyuki Ikezoe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>
#include <time.h>
#include <gtk/gtk.h>

#include "intl.h"
#include "kz-hinadi.h"


static gchar *
kz_hinadi_get_encoding (gchar *src)
{
	gchar *encoding = NULL;
	gchar **lines = NULL;
	gchar *begin_pos, *end_pos, *line_end;;
	
	g_return_val_if_fail(src, NULL);

	lines = g_strsplit(src, "\r\n\r\n", 1);

	begin_pos =  strstr(lines[0], "charset=");
	if (begin_pos)
	{
		begin_pos += 8;
		line_end = strstr(begin_pos, "\r\n");
		end_pos = g_strstr_len(begin_pos, line_end - begin_pos, ";");
		if (end_pos)
			encoding = g_strndup(begin_pos, end_pos - begin_pos);
		else
			encoding = g_strndup(begin_pos, line_end - begin_pos);
	}
	else
		encoding = g_strdup("EUC-JP"); /* default value */

	g_strfreev(lines);
	return encoding;
}


void
kz_hinadi_parse_from_string (KzRemoteBookmark *rbookmark,
			     const gchar *buffer, guint length,
			     GError **error)
{
	gchar *dest = NULL;
	guint read_len, dest_len;
	gchar **lines    = NULL;
	gchar **elements = NULL;
	guint line_number = 1;
	guint element_number;
	gchar timestr[20];
	gchar *encoding;

	g_return_if_fail(buffer != NULL);

	/* convert to utf-8 encoding */
	encoding = kz_hinadi_get_encoding((gchar*)buffer);	
	dest = g_convert(buffer, length, "UTF-8", encoding,
			 &read_len, &dest_len, NULL);
	g_free(encoding);
	
	lines = g_strsplit(dest, "\r\n\r\n", -1);
	
	if (lines[1] == NULL)
	{
		g_strfreev(lines);
		return;
	}

	while(lines[line_number+1] != NULL)
	{
		KzBookmark *item;
		struct tm t;
		
		elements = g_strsplit(lines[line_number], "\r\n", 20);
		element_number = 0;
		item = kz_bookmark_new();
		while(elements[element_number+1] != NULL)
		{
			switch (elements[element_number][0])
			{
			 case 'L':
			 case 'l': /* Last-Modified */
				if(elements[element_number][13] == ':')
				{
					strptime(elements[element_number]+15,
						 "%a, %d %b %Y %H:%M:%S %z", &t);
					kz_bookmark_set_last_modified(item, (guint)mktime(&t));
					strftime(timestr, 20, "%Y/%m/%d %H:%M", &t);
					kz_bookmark_set_description(item, timestr);
				}
				break;
			 case 'T':
			 case 't': /* Title */
				kz_bookmark_set_title(item, elements[element_number] + 6);
				break;
			 case 'U': 
			 case 'u':/* URL */
				kz_bookmark_set_link(item, elements[element_number] + 4);
				break;
			 default:
				break;
			}
			element_number++;
		}
		g_strfreev(elements);
		kz_bookmark_append(KZ_BOOKMARK(rbookmark), item);
		g_object_unref(G_OBJECT(item));
		line_number++;
	}

	g_strfreev(lines);
	
	/* sorting */
	kz_bookmark_sort(KZ_BOOKMARK(rbookmark),
			 KZ_BOOKMARK_SORT_LAST_MODIFIED);
}
