/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Copyright (C) 2003 Hiroyuki Ikezoe
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __KZ_REMOTE_BOOKMARK_H__
#define __KZ_REMOTE_BOOKMARK_H__

#include <glib-object.h>
#include "kz-bookmark.h"

G_BEGIN_DECLS

#define KZ_TYPE_REMOTE_BOOKMARK			(kz_remote_bookmark_get_type ())
#define KZ_REMOTE_BOOKMARK(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), KZ_TYPE_REMOTE_BOOKMARK, KzRemoteBookmark))
#define KZ_REMOTE_BOOKMARK_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass),  KZ_TYPE_REMOTE_BOOKMARK, KzRemoteBookmarkClass))
#define KZ_IS_REMOTE_BOOKMARK(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), KZ_TYPE_REMOTE_BOOKMARK))
#define KZ_IS_REMOTE_BOOKMARK_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass),  KZ_TYPE_REMOTE_BOOKMARK))
#define KZ_REMOTE_BOOKMARK_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj),  KZ_TYPE_REMOTE_BOOKMARK, KzRemoteBookmarkClass))

typedef struct _KzRemoteBookmark      KzRemoteBookmark;
typedef struct _KzRemoteBookmarkClass KzRemoteBookmarkClass;

typedef enum {
	KZ_REMOTE_BOOKMARK_NORMAL,
	KZ_REMOTE_BOOKMARK_LOADING,
} KzRemoteBookmarkState;

struct _KzRemoteBookmark
{
	KzBookmark parent;

	/* update timer id */
	guint timer_id;
	/* update interval minutes */
	guint  update_interval;

	KzRemoteBookmarkState state;
};

struct _KzRemoteBookmarkClass
{
	KzBookmarkClass parent_class;

	/* -- signals -- */
	void (*update_start)         (KzRemoteBookmark *rbookmark);
	void (*update_completed)     (KzRemoteBookmark *rbookmark);
};

GType    kz_remote_bookmark_get_type     (void);
KzRemoteBookmark
	*kz_remote_bookmark_new          (gchar *location,
					  gchar *title,
					  guint  update_interval);
void     kz_remote_bookmark_start        (KzRemoteBookmark *rbookmark);

void     kz_remote_bookmark_force_update (KzRemoteBookmark *rbookmark);

gboolean kz_remote_bookmark_update       (KzRemoteBookmark *rbookmark);
void     kz_remote_bookmark_dispose      (GObject *object);

void     kz_remote_bookmark_set_interval (KzRemoteBookmark *rbookmark,
					  guint   interval); /* [min] */

G_END_DECLS

#endif /* __KZ_REMOTE_BOOKMARK_H__ */
