/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Copyright (C) 2003 Takuro Ashie
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: kz-xbel.c,v 1.14 2003/10/28 00:24:10 makeinu Exp $
 */

#include "kz-xbel.h"

#include <string.h>
#include "gobject-utils.h"

static void     kz_xbel_class_init         (KzXBELClass *klass);
static void     kz_xbel_init               (KzXBEL      *xbel);
static GObject *kz_xbel_constructor        (GType                  type,
					    guint                  n_props,
					    GObjectConstructParam *props);
static void     kz_xbel_dispose            (GObject     *object);

static void     kz_xbel_build_tree         (KzXBEL      *xbel);
static void     kz_xbel_insert_child       (KzBookmark  *bookmark,
					    KzBookmark  *child,
					    KzBookmark  *sibling);
static void     kz_xbel_remove_child       (KzBookmark  *bookmark,
					    KzBookmark  *child);
static void     kz_xble_insert_xml_node    (KzBookmark  *bookmark,
					    KzBookmark  *parent,
					    KzBookmark  *sibling);
static void     kz_xble_remove_xml_node    (KzBookmark  *bookmark);
static void     kz_xble_connect_signals    (KzBookmark  *bookmark);
static void     kz_xble_disconnect_signals (KzBookmark  *bookmark);
static void     cb_bookmark_insert_child   (KzBookmark  *bookmark,
					    KzBookmark  *child,
					    KzBookmark  *sibling);
static void     cb_bookmark_remove_child   (KzBookmark  *bookmark,
					    KzBookmark  *child);
static void     cb_bookmark_notify         (GObject     *object,
					    GParamSpec  *spec);


static KzBookmarkClass *parent_class = NULL;
static GQuark node_quark = 0;
static GQuark building_quark = 0;


#define BOOKMARK_SET_BUILDING(b) \
	g_object_set_qdata(G_OBJECT(b), building_quark, GINT_TO_POINTER(TRUE))
#define BOOKMARK_UNSET_BUILDING(b) \
	g_object_set_qdata(G_OBJECT(b), building_quark, GINT_TO_POINTER(FALSE))
#define BOOKMARK_IS_BUILDING(b) \
	GPOINTER_TO_INT(g_object_get_qdata(G_OBJECT(b), building_quark))


KZ_OBJECT_GET_TYPE(kz_xbel, "KzXBEL", KzXBEL,
		   kz_xbel_class_init, kz_xbel_init,
		   KZ_TYPE_BOOKMARK)


static void
kz_xbel_class_init (KzXBELClass *klass)
{
	GObjectClass *object_class;
	KzBookmarkClass *bookmark_class;

	parent_class   = g_type_class_peek_parent (klass);
	object_class   = (GObjectClass *) klass;
	bookmark_class = (KzBookmarkClass *) klass;

	object_class->constructor = kz_xbel_constructor;
	object_class->dispose     = kz_xbel_dispose;

	bookmark_class->insert_child = kz_xbel_insert_child;
	bookmark_class->remove_child = kz_xbel_remove_child;

	node_quark     = g_quark_from_string("KzXBEL::xmlnode");
	building_quark = g_quark_from_string("KzXBEL::building");
}


static void
kz_xbel_init (KzXBEL *xbel)
{
	xbel->xml = kz_xml_new();
	KZ_BOOKMARK(xbel)->flags |= KZ_BOOKMARK_HAS_LOCATION;
}


static GObject*
kz_xbel_constructor (GType                  type,
                     guint                  n_props,
                     GObjectConstructParam *props)
{
	GObject *object;
	GObjectClass *klass = G_OBJECT_CLASS(parent_class);

	object = klass->constructor(type, n_props, props);

	g_object_set(object, "type", KZ_BOOKMARK_PURE_FOLDER, NULL);

	return object;
}


KzXBEL *
kz_xbel_new (void)
{
	KzXBEL *xbel = KZ_XBEL(g_object_new(KZ_TYPE_XBEL, NULL));
	return xbel;
}


KzXBEL *
kz_xbel_new_from_file (const gchar *filename)
{
	KzXBEL *xbel = KZ_XBEL(g_object_new(KZ_TYPE_XBEL,
					    "location", filename,
					    NULL));
	if (!kz_xml_load(xbel->xml, filename)
	    || !kz_xml_get_root_element(xbel->xml))
	{
		KzXMLNode *node, *title_node, *title, *space;

		node = kz_xml_element_node_new("xbel");
		kz_xml_node_set_attr(node, "version", "1.0");
		kz_xml_node_set_attr(node, "folded", "node");	
		/* kz_xml_node_set_attr(node, "id", "????????"); */
		kz_xml_node_append_child(xbel->xml->root, node);

		space = kz_xml_text_node_new("\n");
		kz_xml_node_append_child(node, space);

		{
			title_node = kz_xml_element_node_new("title");
			kz_xml_node_append_child(node, title_node);

			title = kz_xml_text_node_new("Bookmarks");
			kz_xml_node_append_child(title_node, title);
		}

		space = kz_xml_text_node_new("\n");
		kz_xml_node_append_child(node, space);
	}

	kz_xbel_build_tree(xbel);
	return xbel;
}


static void
kz_xbel_dispose (GObject *object)
{
	KzXBEL *xbel = KZ_XBEL(object);

	if (xbel->xml)
		g_object_unref(xbel->xml);
	xbel->xml = NULL;

	if (G_OBJECT_CLASS (parent_class)->dispose)
		G_OBJECT_CLASS (parent_class)->dispose(object);
}


static void
parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
{
	KzXMLNode *node;

	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));

	if (!kz_xml_node_is_element(parent)) return;

	for (node = kz_xml_node_first_child(parent);
	     node;
	     node = kz_xml_node_next(node))
	{
		if (!kz_xml_node_is_element(node)) continue;

		if (kz_xml_node_name_is(node, "title"))
		{
			gchar *title;

			if (kz_bookmark_get_title(bookmark))
			{
				g_warning("title element is duplicated!");
				continue;
			}
			title = kz_xml_node_to_str(node);
			kz_bookmark_set_title(bookmark, title);
			g_free(title);
		}
		else if (kz_xml_node_name_is(node, "desc"))
		{
			gchar *desc;

			if (kz_bookmark_get_description(bookmark))
			{
				g_warning("desc element is duplicated!");
				continue;
			}
			desc = kz_xml_node_to_str(node);
			kz_bookmark_set_description(bookmark, desc);
			g_free(desc);
		}
		else if (kz_xml_node_name_is(node, "folder"))
		{
			KzBookmark *folder = kz_bookmark_pure_folder_new();
			const gchar *folded_str;
			gboolean folded;

			folded_str = kz_xml_node_get_attr(node, "folded");
			folded = folded_str && !strcmp(folded_str, "yes");
			kz_bookmark_set_folded(folder, folded);

			BOOKMARK_SET_BUILDING(folder);
			g_object_set_qdata(G_OBJECT(folder),
					   node_quark, node);
			kz_bookmark_append(bookmark, folder);
			parse_child_node(folder, node);
			BOOKMARK_UNSET_BUILDING(folder);
			g_object_unref(folder);
		}
		else if (kz_xml_node_name_is(node, "bookmark"))
		{
			KzBookmark *child_bookmark = kz_bookmark_new();
			const gchar *uri = kz_xml_node_get_attr(node, "href");

			BOOKMARK_SET_BUILDING(bookmark);
			g_object_set_qdata(G_OBJECT(child_bookmark),
					   node_quark, node);
			kz_bookmark_set_link(child_bookmark, uri);
			kz_bookmark_append(bookmark, child_bookmark);
			parse_child_node(child_bookmark, node);
			BOOKMARK_UNSET_BUILDING(bookmark);
			g_object_unref(child_bookmark);
		}
		else if (kz_xml_node_name_is(node, "separator"))
		{
			KzBookmark *separator = kz_bookmark_separator_new();

			BOOKMARK_SET_BUILDING(separator);
			g_object_set_qdata(G_OBJECT(separator),
					   node_quark, node);
			kz_bookmark_append(bookmark, separator);
			BOOKMARK_UNSET_BUILDING(separator);
			g_object_unref(separator);
		}
		else if (kz_xml_node_name_is(node, "alias"))
		{
			g_warning("KzXBEL::alias element is not supported yet."
				  "Please implement me!");
		}
	}
}


static void
kz_xbel_build_tree (KzXBEL *xbel)
{
	KzXMLNode *node;

	g_return_if_fail(KZ_IS_XBEL(xbel));

	node = kz_xml_get_root_element (xbel->xml);
	if (!node) return;
	if (!kz_xml_node_name_is(node, "xbel")) return;

	BOOKMARK_SET_BUILDING(xbel);
	g_object_set_qdata(G_OBJECT(xbel),
			   node_quark, node);
	parse_child_node(KZ_BOOKMARK(xbel), node);
	BOOKMARK_UNSET_BUILDING(xbel);
}


static void
xml_node_append_title (KzXMLNode *node, KzBookmark *bookmark)
{
	const gchar *title = kz_bookmark_get_title(bookmark);
	KzXMLNode *title_node, *text_node, *space;

	/* space */
	space = kz_xml_text_node_new("\n");
	kz_xml_node_append_child(node, space);

	/* title */
	title_node = kz_xml_element_node_new("title");
	kz_xml_node_append_child(node, title_node);
	text_node = kz_xml_text_node_new(title);
	kz_xml_node_append_child(title_node, text_node);

	/* space */
	space = kz_xml_text_node_new("\n");
	kz_xml_node_append_child(node, space);
}


static KzXMLNode *
create_xml_node (KzBookmark *bookmark)
{
	KzXMLNode *node = NULL;

	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), NULL);

	if (kz_bookmark_is_separator(bookmark))
	{
		node = kz_xml_element_node_new("separator");
	}
	else if (kz_bookmark_is_folder(bookmark))
	{
		node = kz_xml_element_node_new("folder");
		xml_node_append_title (node, bookmark);
	}
	else
	{
		const gchar *uri = kz_bookmark_get_link(bookmark);

		node = kz_xml_element_node_new("bookmark");
		if (uri)
			kz_xml_node_set_attr(node, "href", uri);
		xml_node_append_title (node, bookmark);
	}

	return node;
}


static void
kz_xble_connect_signals (KzBookmark *bookmark)
{
	if (kz_bookmark_is_folder(bookmark))
	{
		g_signal_connect(G_OBJECT(bookmark), "insert-child",
				 G_CALLBACK(cb_bookmark_insert_child), NULL);
		g_signal_connect(G_OBJECT(bookmark), "remove-child",
				 G_CALLBACK(cb_bookmark_remove_child), NULL);
	}

	g_signal_connect(G_OBJECT(bookmark), "notify",
			 G_CALLBACK(cb_bookmark_notify), NULL);

	if (kz_bookmark_is_folder(bookmark))
	{
		GList *node, *children;

		children = kz_bookmark_get_children(bookmark);
		for (node = children; node; node = g_list_next(node))
		{
			KzBookmark *child = node->data;
			kz_xble_connect_signals(child);
		}
		g_list_free(children);
	}
}


static void
kz_xble_disconnect_signals (KzBookmark *bookmark)
{
	if (kz_bookmark_is_folder(bookmark))
	{
		g_signal_handlers_disconnect_by_func
			(G_OBJECT(bookmark),
			 G_CALLBACK(cb_bookmark_insert_child), NULL);
		g_signal_handlers_disconnect_by_func
			(G_OBJECT(bookmark),
			 G_CALLBACK(cb_bookmark_remove_child), NULL);
	}

	g_signal_handlers_disconnect_by_func
		(G_OBJECT(bookmark),
		 G_CALLBACK(cb_bookmark_notify), NULL);

	if (kz_bookmark_is_folder(bookmark))
	{
		GList *node, *children;

		children = kz_bookmark_get_children(bookmark);
		for (node = children; node; node = g_list_next(node))
		{
			KzBookmark *child = node->data;
			kz_xble_disconnect_signals(child);
		}
		g_list_free(children);
	}
}


static void
cb_bookmark_insert_child (KzBookmark *bookmark,
		 KzBookmark *child, KzBookmark *sibling)
{
	kz_xble_insert_xml_node(child, bookmark, sibling);
	kz_xble_connect_signals(child);
}


static void
cb_bookmark_remove_child (KzBookmark *bookmark, KzBookmark *child)
{
	kz_xble_disconnect_signals(child);
	kz_xble_remove_xml_node(child);
}


static void
xml_node_set_title (KzXMLNode *parent, const gchar *title)
{
	KzXMLNode *title_node = NULL, *node, *child;

	g_return_if_fail(parent);

	node = kz_xml_node_first_child(parent);

	/* find title node */
	for (; node; node = kz_xml_node_next(node))
	{
		if (kz_xml_node_name_is(node, "title"))
		{
			title_node = node;
			break;
		}
	}

	g_return_if_fail(title_node);

	/* remove old title */
	child = kz_xml_node_first_child(title_node);
	while(child)
	{
		KzXMLNode *next = kz_xml_node_next(child);

		child = kz_xml_node_remove_child(title_node, child);
		kz_xml_node_unref(child);
		child = next;
	}

	/* set new title */
	child = kz_xml_text_node_new(title);
	kz_xml_node_append_child(title_node, child);
}


static void
xml_node_set_description (KzXMLNode *parent, const gchar *desc)
{
	KzXMLNode *desc_node = NULL, *node, *child;

	g_return_if_fail(parent);

	node = kz_xml_node_first_child(parent);

	/* find desc node */
	for (; node; node = kz_xml_node_next(node))
	{
		if (kz_xml_node_name_is(node, "desc"))
		{
			desc_node = node;
			break;
		}
	}

	/* remove old title, or create new desc node */
	if (desc_node)
	{
		child = kz_xml_node_first_child(node);
		while(child)
		{
			KzXMLNode *next = kz_xml_node_next(child);

			child = kz_xml_node_remove_child(node, child);
			kz_xml_node_unref(child);
			child = next;
		}
	}
	else
	{
		KzXMLNode *space;

		desc_node = kz_xml_element_node_new("desc");
		kz_xml_node_append_child(parent, desc_node);

		space = kz_xml_text_node_new("\n");
		kz_xml_node_append_child(parent, space);
	}

	/* set new description */
	child = kz_xml_text_node_new(desc);
	kz_xml_node_append_child(desc_node, child);
}


static void
cb_bookmark_notify (GObject *object, GParamSpec *pspec)
{
	KzBookmark *bookmark;
	KzXMLNode *node;
	const gchar *prop;
        GValue value = { 0 };

	g_return_if_fail(KZ_IS_BOOKMARK(object));
	bookmark = KZ_BOOKMARK(object);

	if (BOOKMARK_IS_BUILDING(bookmark)) return;

	node = g_object_get_qdata(G_OBJECT(bookmark), node_quark);
	g_return_if_fail(node);
	g_return_if_fail(kz_xml_node_name_is(node, "bookmark") ||
			 kz_xml_node_name_is(node, "folder"));

	prop = g_param_spec_get_name(pspec);
	g_return_if_fail(prop);

        g_value_init(&value, G_PARAM_SPEC_VALUE_TYPE(pspec));
        g_object_get_property(object, prop, &value);

	/*
	 * FIXME! We should use hash table.
	 */
	if (!strcmp(prop, "title"))
	{
		gchar *title = g_value_dup_string(&value);

		xml_node_set_title(node, title);
		g_free(title);
	}
	else if (!strcmp(prop, "link"))
	{
		gchar *uri = g_value_dup_string(&value);

		kz_xml_node_set_attr(node, "href", uri);
		g_free(uri);
	}
	else if (!strcmp(prop, "description"))
	{
		gchar *desc = g_value_dup_string(&value);

		xml_node_set_description(node, desc);
		g_free(desc);
	}
}


static void
kz_xbel_insert_child (KzBookmark *bookmark,
		      KzBookmark *child, KzBookmark *sibling)
{
	cb_bookmark_insert_child(bookmark, child, sibling);
	parent_class->insert_child(bookmark, child, sibling);
}


static void
kz_xbel_remove_child (KzBookmark *bookmark, KzBookmark *child)
{
	parent_class->remove_child(bookmark, child);
	cb_bookmark_remove_child(bookmark, child);
}


void
kz_xbel_save (KzXBEL *xbel)
{
	const gchar *location;;

	g_return_if_fail(KZ_IS_XBEL(xbel));

	location = kz_bookmark_get_location(KZ_BOOKMARK(xbel));
	if (location && *location)
		kz_xml_save(xbel->xml, location);
}


static void
kz_xble_insert_xml_node (KzBookmark *bookmark,
			 KzBookmark *parent, KzBookmark *sibling)
{
	KzXMLNode *node, *space;
	KzXMLNode *parent_node, *sibling_node = NULL;

	g_return_if_fail(KZ_IS_BOOKMARK(parent));
	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
	g_return_if_fail(!sibling || KZ_IS_BOOKMARK(sibling));

	node = g_object_get_qdata(G_OBJECT(bookmark), node_quark);
	if (node) return;

	parent_node = g_object_get_qdata(G_OBJECT(parent),
					 node_quark);
	if (!parent_node)
	{
		g_return_if_fail(KZ_IS_XBEL(parent));
		parent_node = kz_xml_get_root_element(KZ_XBEL(parent)->xml);
	}

	if (sibling)
		sibling_node = g_object_get_qdata(G_OBJECT(sibling),
						  node_quark);

	node = create_xml_node(bookmark);
	g_object_set_qdata(G_OBJECT(bookmark), node_quark, node);

	kz_xml_node_insert_before(parent_node, node, sibling_node);
	space = kz_xml_text_node_new("\n");
	kz_xml_node_insert_before(parent_node, space, kz_xml_node_next(node));

	if (kz_bookmark_is_folder(bookmark))
	{
		GList *node, *children;

		children = kz_bookmark_get_children(bookmark);
		for (node = children; node; node = g_list_next(node))
		{
			KzBookmark *child = node->data;
			kz_xble_insert_xml_node (child, bookmark, NULL);
		}
		g_list_free(children);
	}
}


static void
kz_xble_remove_xml_node (KzBookmark *bookmark)
{
	KzXMLNode *node, *parent;

	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));

	if (kz_bookmark_is_folder(bookmark))
	{
		GList *node, *children;

		children = kz_bookmark_get_children(bookmark);
		for (node = children; node; node = g_list_next(node))
		{
			KzBookmark *child = node->data;
			kz_xble_remove_xml_node (child);
		}
		g_list_free(children);
	}

	node = g_object_get_qdata(G_OBJECT(bookmark), node_quark);
	if (!node) return;

	parent = kz_xml_node_parent(node);
	if (parent)
		node = kz_xml_node_remove_child(parent, node);

	kz_xml_node_unref(node);
	g_object_set_qdata(G_OBJECT(bookmark), node_quark, NULL);
}
