/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Copyright (C) 2003 Hiroyuki Ikezoe
 *  Copyright (C) 2003 Takuro Ashie <ashie@homa.ne.jp>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: kz-bookmark-menu.c,v 1.5 2003/10/26 14:16:51 makeinu Exp $
 */

/*
 *  TODO:
 *      * Connect to KzBookmark's signals.
 *      * Enable to add sub menus for sub folders recursively.
 *      * Show favicons instead of buttons for normal bookmark.
 *      * Show right click menu.
 */

#include <gtk/gtk.h>

#include "gobject-utils.h"
#include "intl.h"
#include "kz-bookmark-menu.h"
#include "kz-icons.h"
#include "gtk-utils.h"


#define KZ_BOOKMARK_MENU_ITEM_KEY "KzBookmarkMenu::Item"


static void     kz_bookmark_menu_class_init            (KzBookmarkMenuClass *klass);
static void     kz_bookmark_menu_init                  (KzBookmarkMenu *bkmkmenu);
static void     kz_bookmark_menu_dispose               (GObject   *object);

static void     kz_bookmark_menu_load_start            (KzRemoteBookmark *meta,
							KzBookmarkMenu *bkmkmenu);
static void     kz_bookmark_menu_update                (KzRemoteBookmark *meta,
							KzBookmarkMenu *bkmkmenu);
static gboolean kz_bookmark_menu_item_button_press_cb  (GtkWidget *widget,
							GdkEventButton *event,
							KzBookmarkMenu *bkmkmenu);
static gboolean kz_bookmark_menu_all_button_press_cb   (GtkWidget *widget,
							GdkEventButton *event,
							KzBookmarkMenu *bkmkmenu);
static void     kz_bookmark_menu_items_new             (KzWindow *kz,
							KzBookmarkMenu *bkmkmenu);

static gboolean kz_bookmark_menu_press_cb              (GtkWidget *widget,
							GdkEvent *event,
							KzBookmarkMenu *bkmkmenu);
static void     kz_bookmark_menu_update_button_clicked (GtkWidget *widget,
							KzBookmarkMenu *bkmkmenu);


static GtkHBoxClass *parent_class = NULL;

KZ_OBJECT_GET_TYPE(kz_bookmark_menu, "KzBookmarkMenu", KzBookmarkMenu,
		   kz_bookmark_menu_class_init, kz_bookmark_menu_init,
		   GTK_TYPE_HBOX)

static void
kz_bookmark_menu_class_init (KzBookmarkMenuClass *klass)
{
	GObjectClass *gobject_class;

	parent_class = g_type_class_peek_parent (klass);

	gobject_class = (GObjectClass *) klass;

	/* GObject signals */
	gobject_class->dispose = kz_bookmark_menu_dispose;
}

static void
kz_bookmark_menu_init (KzBookmarkMenu *bkmkmenu)
{
	bkmkmenu->kz            = NULL;
	bkmkmenu->bookmark      = NULL;
	bkmkmenu->sub_menu      = NULL;
	bkmkmenu->menu_item     = NULL;
	bkmkmenu->update_button = NULL;
	bkmkmenu->image         = NULL;
}

static void
kz_bookmark_menu_dispose (GObject *object)
{
	KzBookmarkMenu *bkmkmenu;

	g_return_if_fail (KZ_IS_BOOKMARK_MENU(object));

	bkmkmenu = KZ_BOOKMARK_MENU(object);

	bkmkmenu->kz = NULL;

	if (bkmkmenu->bookmark)
	{
		g_signal_handlers_disconnect_by_func(
			G_OBJECT(bkmkmenu->bookmark),
			kz_bookmark_menu_load_start,
			bkmkmenu);
		g_signal_handlers_disconnect_by_func(
			G_OBJECT(bkmkmenu->bookmark),
			kz_bookmark_menu_update,
			bkmkmenu);
		g_object_unref(bkmkmenu->bookmark);
		bkmkmenu->bookmark = NULL;
	}

	if (bkmkmenu->sub_menu)
	{
		g_object_unref(G_OBJECT(bkmkmenu->sub_menu));
		bkmkmenu->sub_menu = NULL;
	}

	if (G_OBJECT_CLASS (parent_class)->dispose)
		G_OBJECT_CLASS (parent_class)->dispose(object);
}

GtkWidget *
kz_bookmark_menu_new (KzWindow *kz, KzBookmark *bookmark)
{
	KzBookmarkMenu *bkmkmenu;
	GtkWidget *button, *image, *hbox, *label, *eventbox;

	g_return_val_if_fail(KZ_IS_WINDOW(kz), NULL);
	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), NULL);

	bkmkmenu = KZ_BOOKMARK_MENU(g_object_new(KZ_TYPE_BOOKMARK_MENU, NULL));
	bkmkmenu->kz = kz;
	bkmkmenu->bookmark = bookmark;
	g_object_ref(bkmkmenu->bookmark);

	gtk_container_set_border_width(GTK_CONTAINER(bkmkmenu), 2);

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(bkmkmenu), hbox);
	gtk_widget_show(hbox);

	/* button */
	if (KZ_IS_REMOTE_BOOKMARK(bkmkmenu->bookmark))
	{
		bkmkmenu->update_button = button = gtk_button_new();
		gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
		gtk_container_set_border_width(GTK_CONTAINER(button), 0);
		g_signal_connect(G_OBJECT(button), "clicked", 
				 G_CALLBACK(kz_bookmark_menu_update_button_clicked),
				 bkmkmenu);
		gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);

		bkmkmenu->image = image
			= gtk_image_new_from_stock(KZ_STOCK_GREEN,
						   KZ_ICON_SIZE_BOOKMARK_MENU);

		gtk_container_add(GTK_CONTAINER(button), image);
		gtk_widget_set_size_request(button, 16, 16);
		gtk_widget_show(image);
		gtk_widget_show(button);
	}
	else
	{
		/* show favicon? */
	}

	/* label */
	eventbox = gtk_event_box_new();
	gtk_box_pack_start(GTK_BOX(hbox), eventbox, FALSE, FALSE, 0);
	gtk_widget_show (eventbox);

	label = gtk_label_new(kz_bookmark_get_title(bookmark));
	g_signal_connect(G_OBJECT(eventbox), "button_press_event",
			 G_CALLBACK(kz_bookmark_menu_press_cb), bkmkmenu);
	gtk_container_add(GTK_CONTAINER(eventbox), label);
	gtk_widget_show (label);

	kz_bookmark_menu_items_new(kz, bkmkmenu);

	gtk_widget_show_all(GTK_WIDGET(bkmkmenu));

	if (KZ_IS_REMOTE_BOOKMARK(bkmkmenu->bookmark))
	{
		g_signal_connect(G_OBJECT(bkmkmenu->bookmark),
				 "update_start", 
				 G_CALLBACK(kz_bookmark_menu_load_start),
				 bkmkmenu);
		g_signal_connect(G_OBJECT(bkmkmenu->bookmark),
				 "update_completed", 
				 G_CALLBACK(kz_bookmark_menu_update),
				 bkmkmenu);

		if (KZ_REMOTE_BOOKMARK(bkmkmenu->bookmark)->state
		    == KZ_REMOTE_BOOKMARK_LOADING)
		{
			kz_bookmark_menu_load_start(
				KZ_REMOTE_BOOKMARK(bkmkmenu->bookmark),
				bkmkmenu);
		}
	}

	return GTK_WIDGET(bkmkmenu);
}

static void
kz_bookmark_menu_items_new (KzWindow *kz, KzBookmarkMenu *bkmkmenu)
{
	GtkWidget *menu_item;
	GList *node, *items;
	KzRemoteBookmark *rbookmark = NULL;
	KzBookmark *item = NULL;
	GtkTooltips *tooltip = NULL;

	g_return_if_fail(KZ_IS_BOOKMARK_MENU(bkmkmenu));

	rbookmark = KZ_REMOTE_BOOKMARK(bkmkmenu->bookmark);

	if (bkmkmenu->sub_menu)
	{
		g_object_unref(bkmkmenu->sub_menu);
	}
	bkmkmenu->sub_menu = gtk_menu_new();
	g_object_ref(G_OBJECT(bkmkmenu->sub_menu));
	gtk_object_sink(GTK_OBJECT(bkmkmenu->sub_menu));

	/* Open all */
	menu_item = gtk_menu_item_new_with_label(_("Open all items"));
	gtk_menu_shell_append(GTK_MENU_SHELL(bkmkmenu->sub_menu),
			      menu_item);
	g_signal_connect(G_OBJECT(menu_item), "button_press_event",
			 G_CALLBACK(kz_bookmark_menu_all_button_press_cb),
			 bkmkmenu);

	/* separator */
	menu_item = gtk_separator_menu_item_new();
	gtk_menu_shell_append(GTK_MENU_SHELL(bkmkmenu->sub_menu),
			      menu_item);

	/* items */
	items = kz_bookmark_get_children(KZ_BOOKMARK(rbookmark));
	for (node = items; node; node = g_list_next(node))
	{
		const gchar *title;

		item = node->data;
		title = kz_bookmark_get_title(item);
		if (!title) continue;

		menu_item = gtk_menu_item_new_with_label(title);
		gtk_menu_shell_append(GTK_MENU_SHELL(bkmkmenu->sub_menu),
				     menu_item);
		g_object_set_data(G_OBJECT(menu_item),
				  KZ_BOOKMARK_MENU_ITEM_KEY,
				  item);
		g_signal_connect(G_OBJECT(menu_item), "button_press_event",
				 G_CALLBACK(kz_bookmark_menu_item_button_press_cb),
				 bkmkmenu);
		tooltip = gtk_tooltips_new();
		gtk_tooltips_set_tip(tooltip, menu_item,
				     kz_bookmark_get_description(item), NULL);
	}
	g_list_free(items);

	gtk_widget_show_all(bkmkmenu->sub_menu);
}

static void 
kz_bookmark_menu_load_start (KzRemoteBookmark *meta, KzBookmarkMenu *bkmkmenu)
{
	g_return_if_fail(KZ_IS_REMOTE_BOOKMARK(meta));
	g_return_if_fail(KZ_IS_BOOKMARK_MENU(bkmkmenu));

	gtk_image_set_from_stock(GTK_IMAGE(bkmkmenu->image),
				 KZ_STOCK_RED,
				 KZ_ICON_SIZE_BOOKMARK_MENU);
}

static void 
kz_bookmark_menu_update (KzRemoteBookmark *rbookmark, KzBookmarkMenu *bkmkmenu)
{
	KzWindow *kz;

	g_return_if_fail(KZ_IS_REMOTE_BOOKMARK(rbookmark));
	g_return_if_fail(KZ_IS_BOOKMARK_MENU(bkmkmenu));
	kz = bkmkmenu->kz;

	kz_bookmark_menu_items_new(kz, bkmkmenu);
	gtk_widget_show_all(GTK_WIDGET(bkmkmenu));

	gtk_image_set_from_stock(GTK_IMAGE(bkmkmenu->image),
				 KZ_STOCK_GREEN,
				 KZ_ICON_SIZE_BOOKMARK_MENU);
}

static gboolean
kz_bookmark_menu_all_button_press_cb (GtkWidget *widget, GdkEventButton *event,
				      KzBookmarkMenu *bkmkmenu)
{
	KzWindow *kz;
	KzRemoteBookmark *rbookmark;
	GList *items, *node;

	g_return_val_if_fail(KZ_IS_BOOKMARK_MENU(bkmkmenu), FALSE);

	kz = bkmkmenu->kz;
	if (!KZ_IS_WINDOW(kz)) return FALSE;

	rbookmark = KZ_REMOTE_BOOKMARK(bkmkmenu->bookmark);
	if (!KZ_IS_REMOTE_BOOKMARK(rbookmark)) return FALSE;

	items = kz_bookmark_get_children(KZ_BOOKMARK(rbookmark));

	for (node = items; node; node = g_list_next(node))
	{
		KzBookmark *item = node->data;
		if (!item) continue;

		kz_window_open_new_tab(kz, kz_bookmark_get_link(item));
	}

	g_list_free(items);

	return TRUE;
}

static gboolean
kz_bookmark_menu_item_button_press_cb (GtkWidget *widget, GdkEventButton *event,
				       KzBookmarkMenu *bkmkmenu)
{
	KzBookmark *item;
	KzWindow *kz;
	const gchar *uri;

	g_return_val_if_fail(KZ_IS_BOOKMARK_MENU(bkmkmenu), FALSE);
	kz = bkmkmenu->kz;
	if (!KZ_IS_WINDOW(kz)) return FALSE;

	item = g_object_get_data(G_OBJECT(widget), KZ_BOOKMARK_MENU_ITEM_KEY);
	g_return_val_if_fail(item, FALSE);

	uri = kz_bookmark_get_link(item);
	if (!uri) return FALSE;

	switch (event->button) {
	case 1:
	case 3:
		kz_window_load_url(kz, uri);
		return TRUE;
		break;
	case 2:
		kz_window_open_new_tab(kz, uri);
		return TRUE;
		break;
	}

	return FALSE;
}

static gboolean
kz_bookmark_menu_press_cb (GtkWidget *widget, GdkEvent *event,
			   KzBookmarkMenu *bkmkmenu)
{
	KzWindow *kz;
	GdkEventButton *event_button;

	kz = bkmkmenu->kz;

	event_button = (GdkEventButton *)event;

	switch(event_button->button)
	{
	case 1:
		/* g_return_val_if_fail(GTK_IS_MENU(bkmkmenu->sub_menu), FALSE); */
		if (!GTK_IS_MENU(bkmkmenu->sub_menu)) return FALSE;

		gtk_menu_popup(GTK_MENU(bkmkmenu->sub_menu), NULL, NULL, 
			       gtkutil_menu_position_under_widget,
			       GTK_WIDGET(bkmkmenu),
			       event_button->button, event_button->time);
		break;
	case 2:
	{
		GList *node, *items;
		KzBookmark *item = NULL;

		items = kz_bookmark_get_children(bkmkmenu->bookmark);
		for (node = items; node; node = g_list_next(node))
		{
			const gchar *uri;
			item = node->data;
			uri = kz_bookmark_get_link(item);
			if (uri)
				kz_window_open_new_tab(kz, uri);
		}
		g_list_free(items);
		break;
	}
	case 3:
		/* kz_window_load_url(kz, bkmkmenu->meta->meta_link); */
		break;
	}

	return FALSE;
}

static void
kz_bookmark_menu_update_button_clicked (GtkWidget *widget, KzBookmarkMenu *bkmkmenu)
{
	g_return_if_fail(KZ_IS_BOOKMARK_MENU(bkmkmenu));

	kz_remote_bookmark_force_update(KZ_REMOTE_BOOKMARK(bkmkmenu->bookmark));
}
